/* 
 * Thomas Pantzer: t_drawobj.cc, v0.1b  8.11.98 0:58 
 * 
 * 
 * Copyright 1998 by Thomas Pantzer
 * 
 * Permission to use, copy, modify, and distribute this software for noncommercial 
 * use and without fee is hereby granted, provided that the above copyright notice 
 * appear in all copies and that both that copyright notice and this permission 
 * notice appear in supporting documentation, and that the name of the Author not be 
 * used in advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission. The Author makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is" 
 * without expressed or implied warranty. 
 * 
 * 
 * pantec@informatik.uni-leipzig.de   (Thomas Pantzer) 
 * 
 */

// Output from p2c 1.21alpha-07.Dec.93, the Pascal-to-C translator
// From input file "/home/pantec/src/objdraw/t_drawobj.p"

#include <stdlib.h>

#include "forms.h"
//#include "xfgc/graphix.h"

#define T_DRAWOBJ_G
#include "t_drawobj.h"

char e_drawtype_str[12][32] = {"basic\0",
                               "group\0",

                               "line\0",
                               "circle\0",
                               "rectangle\0",
                               "polygon\0",
                               "triangle\0",
                               "arc\0",
                               "string\0",

                               "cursor\0",
                               "resistor\0",
                               "arrow\0"
                               };


void t_drawobj::Init(t_colormap *c, t_vector2d *a, t_drawobj *n, int lw)
{
  cmap = c;
  anchor.Init(a);
  next = n;
  width = lw;
  selected = false;
  opaque = true;
  hidden = 1;
  kind = _basic;
}

void t_drawobj::SaveTo(int fd)
{
  char st[512];
  sprintf(st,"\not:%s\tac:(%d,%d)\tlw:%d\tcm(%d,%d,%d)",
            e_drawtype_str[kind],anchor.GetX(),anchor.GetY(),width,cmap->normal,cmap->selected,cmap->hidden);

  write(fd,st,strlen(st));
}

void t_drawobj::SaveAllTo(int fd)
{
   SaveTo(fd);
   
   if (next != NULL)
      next->SaveAllTo(fd);
}

void t_drawobj::SelectAll(char b)
{
  Select(b);
     
  if (next == NULL)
    return;

  next->SelectAll(b);
}

void t_drawobj::Select(char b)
{
  selected = b;
}

void t_drawobj::OpaqueAll(char b)
{
  Opaque(b);
     
  if (next == NULL)
    return;

  next->OpaqueAll(b);
}

void t_drawobj::Opaque(char b)
{
  opaque = b;
}


void t_drawobj::Insert(t_drawobj *p)
{
  if (next == NULL)
    next = p;
  else
    next->Insert(p);
}


void t_drawobj::Delete(t_drawobj *p)
{
  t_drawobj *delp;

  if (next == NULL)
    return;

  if (next == p)
     {
      next = p->next;

      delp = p;
      delp->Done();
      delp->next = NULL;
      
      delete delp;
      delp = NULL;
      return;
     }
  else
    next->Delete(p);
}

void t_drawobj::Unchain(t_drawobj *p)
{
  t_drawobj *delp;

  if (next == NULL)
    return;

  if (next == p)
     {
      next = p->next;
      delp = p;
      delp->next = NULL;  // break loops 
      return;
     }

  next->Unchain(p);
}


void t_drawobj::Draw(color_type ct)
{
     switch (ct)
       {
        case ct_normal: SetColor(cmap->normal);
          break;

        case ct_selected: SetColor(cmap->selected);
          break;

        case ct_hidden: SetColor(cmap->hidden);
          break;

        case ct_opaque: SetColor(cmap->normal);
          break;
       }   
}


void t_drawobj::ShowAll(void)
{
  Show();

  if (next == NULL)
     return;
     
  next->ShowAll();   
}

void t_drawobj::DrawAll(color_type ct)
{
  Draw(ct);

  if (next == NULL)
     return;
     
  next->DrawAll(ct);   
}

void t_drawobj::HideAll(void)
{
  Hide();
  
  if (next == NULL)
     return;
     
  next->HideAll();   
}



/*
void t_drawobj::SetOpaque(void)
{
      int owm = VScr.GetWriteMode();
      VScr.SetWriteMode(XORPut);
      Draw(ct_opaque);
      VScr.SetWriteMode(owm);
}
*/


void t_drawobj::SetOpaqueView(void)
{
   Hide();
   Select(false);
   Opaque(true);
   Show();
}

void t_drawobj::ClearOpaqueView(void)
{
   Hide();
   Opaque(false);
   Show();
}

void t_drawobj::SetSelectedView(void)
{
   Hide();
   Select(true);
   Show();
}

void t_drawobj::ClearSelectedView(void)
{
   Hide();
   Select(false);
   Show();
}


void t_drawobj::Show(void)
{
  if (opaque == true)
     {
      int owm = VScr.GetWriteMode();
      VScr.SetWriteMode(XORPut);
      Draw(ct_opaque);
      VScr.SetWriteMode(owm);
      hidden++;
      hidden %= 2;
     } 
   else
     {
      if (selected == true)
         Draw(ct_selected);
       else
         Draw(ct_normal); 
      hidden = 0;	 
     } 
}


void t_drawobj::Hide(void)
{
  if (opaque == true)
     {
      int owm = VScr.GetWriteMode();
      VScr.SetWriteMode(XORPut);
      Draw(ct_opaque);
      VScr.SetWriteMode(owm);
      hidden++;
      hidden %= 2;
     } 
   else
     {
      Draw(ct_hidden);
      hidden = 1;	 
     } 
}

void t_drawobj::Rotate(t_vector2d *M, t_vector2d &CSR)
{
#ifdef DEBUG
   fprintf(stderr,"t_drawobj::Rotate(0x%08lx)\n",(long)this); 
#endif
   anchor.Rotate(M,CSR);
}


void t_drawobj::RotateAll(t_vector2d *M, t_vector2d &CSR)
{
#ifdef DEBUG
   fprintf(stderr,"t_drawobj::RotateAll(0x%08lx)\n",(long)this); 
#endif
   if (next != NULL)
      next->RotateAll(M,CSR);  

   Rotate(M,CSR);
}  

void t_drawobj::AnchorAdd(t_vector2d *p)
{
    anchor.Add(p);
}

void t_drawobj::Anchor(t_vector2d *p)
{
    t_vector2d V;
    V.Init(p);
    V.Sub(&anchor);   
   
    AnchorAdd(&V);
}


void t_drawobj::NewAnchor(void)
{
}

t_vector2d *t_drawobj::Anchor(void)
{
   NewAnchor(); 
   return &anchor;
}


void t_drawobj::MoveRel(t_vector2d *p)
{
    Hide();
    AnchorAdd(p);
    Show();
}

void t_drawobj::MoveTo(t_vector2d *p)
{
    Hide();
    Anchor(p);
    Show();
}

void t_drawobj::Done()
{
}


double t_drawobj::Distance(t_vector2d *P)
{
   return P->Distance(&anchor);  
}


t_drawobj *t_drawobj::Distance(double min, t_vector2d *P, t_drawobj *MinP)
{
#ifdef DEBUG
   fprintf(stderr,"%s\t",e_drawtype_str[kind]);
#endif   
   
   double d = Distance(P);
   
   if (next == NULL)
      {
       if (d < min)
          return this;
        else
          return MinP;  
      }  
    else
      {
       if (d < min)
          return next->Distance(d,P,this);
        else
          return next->Distance(min,P,MinP);  
      }  
}


int t_drawobj::Count(void)
{
  if (next == NULL)
     return 1;
   else
     return (1 + (next->Count()));  
}

// End.
