/* 
 * Thomas Pantzer: t_ringbuf.cc, v0.1b  6.8.99 16:10 
 * 
 * 
 * Copyright 1999 by Thomas Pantzer
 * 
 * Permission to use, copy, modify, and distribute this software for noncommercial 
 * use and without fee is hereby granted, provided that the above copyright notice 
 * appear in all copies and that both that copyright notice and this permission 
 * notice appear in supporting documentation, and that the name of the Author not be 
 * used in advertising or publicity pertaining to distribution of the software 
 * without specific, written prior permission. The Author makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is" 
 * without expressed or implied warranty. 
 * 
 * 
 * pantec@zorro.informatik.uni-leipzig.de   (Thomas Pantzer) 
 * 
 */



#include "t_ringbuf.h"
#include "utils.h"
#include <stdio.h>
#include <string.h>



void t_ring_buf::Init(int len, int p, int g)
{
	get_size = g;
	put_size = p;
	
	size = ((__max(len,__max(p,g)*2) / get_size) + 1) * get_size;

	real_size = size + __max(p,g) + 1024;
	
	Array = new char [real_size];
	rptr = &Array[0];
	wptr = &Array[0];
}

void t_ring_buf::Done(void)
{
	delete Array;
	Array = NULL;
}


int t_ring_buf::Put_Avail(void)
{
	int a = wptr - rptr;
	if (a <= 0)
	   a += size;
	   
	return a;	   
}

int t_ring_buf::Get_Avail(void)
{
	int a = rptr - wptr;
	if (a < 0)
	   a += size;
	   
	return a;	   
}


char *t_ring_buf::Get(void)
{
	if (Get_Avail() < get_size) 
	   return NULL;

	// OK to read
	char *buf = wptr;
	wptr += get_size;

	if (wptr >= &Array[size])
	   wptr = &Array[0];
	    
	return buf; 
}



void t_ring_buf::Put(char *buf)
{
	memcpy(rptr,buf,put_size);
	rptr += put_size;
	
	if (rptr > &Array[size])
	   {
	    long wrap = rptr - &Array[size];
	    memmove(&Array[0],&Array[size],wrap);
	    
	    rptr = &Array[wrap];
	   }	
}


char *t_ring_buf::Put(void)
{
	if (Put_Avail() < put_size)
	   return NULL;

	char *buf = rptr;
	
	rptr += put_size;
	
	if (rptr > &Array[size])
	   {
	    long wrap = rptr - &Array[size];
//	    memmove(&Array[0],&Array[size],wrap);
	    
	    rptr = &Array[wrap];
	   }	
	   
	return buf;	   
}







